window.Gleam.initSearch([{"doc":"valid","title":"valid","content":"# Validator\n\n![CI](https://github.com/sporto/gleam-validator/workflows/test/badge.svg?branch=main)\n\nA validation library for [Gleam](https://gleam.run/).\n\nAPI Docs: &lt;https://hexdocs.pm/gleam_validator&gt;.\n\nThis library follows the principle [Parse don&#39;t validate](https://lexi-lambda.github.io/blog/2019/11/05/parse-don-t-validate/).\n\n## Usage\n\nYou start with an input type and validate into an output type. These two types can be different. For example:\n\n```haskell\ntype UserInput { UserInput(name: Option(String), age: Int) }\n\ntype ValidUser { ValidUser(name: String, age: Int) }\n```\n\nThen you create a validator like:\n\n```haskell\nimport validator\nimport validator/common.{ValidatorResult}\nimport validator/int\nimport validator/option\n\nfn user_validator(user: UserInput) -&gt; ValidatorResult(ValidUser, String) {\n  validator.build2(ValidUser)\n  |&gt; validator.validate(user.name, option.is_some(&quot;Please provide a name&quot;))\n  |&gt; validator.validate(user.age, int.min(13, &quot;Must be at least 13 years old&quot;))\n}\n```\n\nAnd run it:\n\n```rust\ncase user_validator(input) {\n  Ok(valid_user) -&gt; ...\n  Error(tuple(first_error, all_errors)) -&gt; ...\n}\n```\n\n## Error type\n\nErrors can be your own type e.g.\n\n```haskell\nimport validator\nimport validator/common.{ValidatorResult}\nimport validator/int\nimport validator/option\n\ntype Error {\n  ErrorEmptyName,\n  ErrorTooYoung,\n}\n\nfn user_validator(user: UserInput) -&gt; ValidatorResult(ValidUser, String) {\n  validator.build2(ValidUser)\n  |&gt; validator.validate(user.name, option.is_some(ErrorEmptyName))\n  |&gt; validator.validate(user.age, int.min(13, ErrorTooYoung))\n}\n```\n\n## ValidatorResult\n\n`ValidatorResult(valid, error)` is an alias for `Result(valid, tuple(error, List(error)))`\n\nThe `Ok` branch has the valid output.\n\nThe `Error` branch has a tuple `tuple(error, List(error))`.\nThe first value is the first error. The second value is a list with all errors (including the first).\n\n## Validators\n\nSee the [API Docs](https://hexdocs.pm/gleam_validator/) for the list of included validators.\n\n## Custom property validator\n\nA property validator has two components:\n\n- The error to return\n- A function that transforms the property if successful (`fn(input) -&gt; Option(output)`)\n\nExample:\n\n```rust\nimport gleam/option.{None, Option, Some}\nimport validator\n\nfn bigger_than_10(num: Int) -&gt; Option(num) {\n  case num &gt; 10 {\n    True -&gt;\n      Some(num)\n    False -&gt;\n      None\n  }\n}\n\nlet custom = validator.custom(&quot;Must be bigger than 10&quot;, bigger_than_10)\n\nlet validator = fn(form: FormInput) {\n  validator.build1(ValidForm)\n  |&gt; validator.validate(form.quantity, custom)\n}\n```\n\n## Examples\n\nSee [the tests](https://github.com/sporto/gleam-validator/blob/main/test/validator_test.gleam) for many examples\n\n## Test\n\n```sh\nrebar3 compile\nrebar3 eunit\n```\n\n## Installation\n\nThis package can be installed by adding `gleam_validator` to your `rebar.config` dependencies:\n\n```erlang\n{deps, [\n    gleam_validator\n]}.\n```\n","url":"index.html"},{"doc":"validator/common","title":"Errors","content":"pub type Errors(error) =\n  #(error, List(error))\n Error type returned by the validator.\n\n This is a # with the first error and a list of all errors.\n The list includes the first error.\n\nSynonyms:\nvalidator/common.Errors\nvalidator/common Errors","url":"validator/common.html#Errors"},{"doc":"validator/common","title":"Validator","content":"pub type Validator(input, output, error) =\n  fn(input) -&gt; ValidatorResult(output, error)\n A Validator is a function that takes an input and\n returns a ValidatorResult\n\nSynonyms:\nvalidator/common.Validator\nvalidator/common Validator","url":"validator/common.html#Validator"},{"doc":"validator/common","title":"ValidatorResult","content":"pub type ValidatorResult(output, error) =\n  Result(output, Errors(error))\n\n\nSynonyms:\nvalidator/common.ValidatorResult\nvalidator/common ValidatorResult","url":"validator/common.html#ValidatorResult"},{"doc":"validator/common","title":"custom","content":"pub fn custom(\n  error: a,\n  check: fn(b) -&gt; Option(c),\n) -&gt; fn(b) -&gt; Result(c, #(a, List(a)))\n Create a custom validator, see documentation in root module\nSynonyms:\nvalidator/common.custom\nvalidator/common custom","url":"validator/common.html#custom"},{"doc":"validator/common","title":"validator/common","content":"","url":"validator/common.html"},{"doc":"validator","title":"all","content":"pub fn all(\n  validators: List(fn(a) -&gt; Result(a, #(b, List(b)))),\n) -&gt; fn(a) -&gt; Result(a, #(b, List(b)))\n Validate a value using a list of validators.\n This runs all the validators in the list.\n\n The initial input is passed to all validators.\n All these validators must have the same input and output types.\n\n Returns Ok when all validators pass.\n Returns Error when any validator fails. Error will have all failures.\n\n ## Example\n\n\tlet name_validator = v.all([\n\t\tv_string.is_not_empty(&quot;Empty&quot;),\n\t\tv_string.min_length(&quot;&gt;=3&quot;, 3),\n\t\tv_string.max_length(&quot;&lt;=10&quot;, 10)\n\t])\n\n\tlet validator = fn(person: Person) {\n\t\tv.build1(person)\n\t\t|&gt; v.validate(person.name, name_validator)\n\t}\nSynonyms:\nvalidator.all\nvalidator all","url":"validator.html#all"},{"doc":"validator","title":"and","content":"pub fn and(\n  validator1: fn(a) -&gt; Result(b, #(c, List(c))),\n  validator2: fn(b) -&gt; Result(d, #(c, List(c))),\n) -&gt; fn(a) -&gt; Result(d, #(c, List(c)))\n Compose validators\n\n Run the first validator and if successful then the second.\n Only returns the first error.\n\n ## Example\n\n\tlet name_validator = v_string.is_not_empty(&quot;Empty&quot;)\n\t|&gt; v.and(v_string.min_length(&quot;Must be at least six&quot;, 6))\nSynonyms:\nvalidator.and\nvalidator and","url":"validator.html#and"},{"doc":"validator","title":"build1","content":"pub fn build1(constructor: a) -&gt; Result(a, b)\n Build a validator for a type that has one attribute\n\n ## Example\n\n\ttype Person { Person(name: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tv.build1(person)\n\t\t|&gt; v.validate(person.name, name_validator)\n\t}\nSynonyms:\nvalidator.build1\nvalidator build1","url":"validator.html#build1"},{"doc":"validator","title":"build2","content":"pub fn build2(\n  constructor: fn(a, b) -&gt; c,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; c, d)\n Build a validator for a type that has two attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int) }\n\n\tlet validator = fn(person: Person) {\n\t\tv.build2(person)\n\t\t|&gt; v.validate(person.name, name_validator)\n\t\t|&gt; v.validate(person.age, ...)\n\t}\nSynonyms:\nvalidator.build2\nvalidator build2","url":"validator.html#build2"},{"doc":"validator","title":"build3","content":"pub fn build3(\n  constructor: fn(a, b, c) -&gt; d,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; d, e)\n Build a validator for a type that has three attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int, email: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tv.build3(person)\n\t\t|&gt; v.validate(person.name, name_validator)\n\t\t|&gt; v.validate(person.age, ...)\n\t\t|&gt; v.validate(person.email, ...)\n\t}\nSynonyms:\nvalidator.build3\nvalidator build3","url":"validator.html#build3"},{"doc":"validator","title":"build4","content":"pub fn build4(\n  constructor: fn(a, b, c, d) -&gt; e,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; e, f)\n Build a validator for a type that has four attributes\nSynonyms:\nvalidator.build4\nvalidator build4","url":"validator.html#build4"},{"doc":"validator","title":"build5","content":"pub fn build5(\n  constructor: fn(a, b, c, d, e) -&gt; f,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; f, g)\n Build a validator for a type that has five attributes\nSynonyms:\nvalidator.build5\nvalidator build5","url":"validator.html#build5"},{"doc":"validator","title":"build6","content":"pub fn build6(\n  constructor: fn(a, b, c, d, e, f) -&gt; g,\n) -&gt; Result(\n  fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; g,\n  h,\n)\n Build a validator for a type that has six attributes\nSynonyms:\nvalidator.build6\nvalidator build6","url":"validator.html#build6"},{"doc":"validator","title":"custom","content":"pub fn custom(\n  error: a,\n  check: fn(b) -&gt; Option(c),\n) -&gt; fn(b) -&gt; Result(c, #(a, List(a)))\n Create a custom validator\n\n A custom validator has two attributes:\n\n - The error\n - A check function\n\n The check function is a function that takes an `input` and returns `Option(output)`\n\n ## Example\n\n\tlet must_be_sam = fn(name: String) -&gt; Option(String) {\n\t\tcase name == &quot;Sam&quot; {\n\t\t\tTrue -&gt; Some(name)\n\t\t\tFalse -&gt; None\n\t\t}\n\t}\n\n\tlet validator = fn(person: Person) {\n\t\tv.build1(Person)\n\t\t|&gt; v.validate(person.name, v.custom(&quot;Not Sam&quot;, must_be_sam))\n\t}\nSynonyms:\nvalidator.custom\nvalidator custom","url":"validator.html#custom"},{"doc":"validator","title":"keep","content":"pub fn keep(\n  accumulator: Result(fn(a) -&gt; b, #(c, List(c))),\n  value: a,\n) -&gt; Result(b, #(c, List(c)))\n Keep a value as is.\n\n ## Example\n\n\tfn person_validor(person: Person) {\n\t\tv.build2(Person)\n\t\t\t|&gt; v.validate(person.name, ...)\n\t\t\t|&gt; v.keep(person.age)\n\t}\n\nSynonyms:\nvalidator.keep\nvalidator keep","url":"validator.html#keep"},{"doc":"validator","title":"validate","content":"pub fn validate(\n  accumulator: Result(fn(a) -&gt; b, #(c, List(c))),\n  value: d,\n  validator: fn(d) -&gt; Result(a, #(c, List(c))),\n) -&gt; Result(b, #(c, List(c)))\n Validate an attribute.\n\n ## Example\n\n\tlet validator = fn(person: Person) {\n\t\tv.build1(Person)\n\t\t|&gt; v.validate(person.name, v_string.is_not_empty(ErrorEmpty))\n\t}\n\nSynonyms:\nvalidator.validate\nvalidator validate","url":"validator.html#validate"},{"doc":"validator","title":"whole","content":"pub fn whole(\n  validator: fn(a) -&gt; Result(a, b),\n) -&gt; fn(Result(a, #(b, List(b)))) -&gt; Result(a, #(b, List(b)))\n Validate a structure as a whole.\n\n Sometimes we need to validate a property in relation to another.\n\n This function requires a check function like:\n\n\tfn(a) -&gt; Result(a, error)\n\n ## Example\n\n\tlet strengh_and_level_validator = fn(c: Character) {\n\t\tcase c.level &gt; c.strength {\n\t\t\tTrue -&gt; Error(error)\n\t\t\tFalse -&gt; Ok(c)\n\t\t}\n\t}\n\n\tlet validator = fn(c: Character) {\n\t\tv.build2(Character)\n\t\t|&gt; v.validate(c.level, v_int.min(&quot;Level must be more that zero&quot;, 1))\n\t\t|&gt; v.validate(c.strength, v_int.min(&quot;Strength must be more that zero&quot;, 1))\n\t\t|&gt; v.whole(strengh_and_level_validator)\n\t}\n\nSynonyms:\nvalidator.whole\nvalidator whole","url":"validator.html#whole"},{"doc":"validator","title":"validator","content":"","url":"validator.html"},{"doc":"validator/int","title":"max","content":"pub fn max(\n  error: a,\n  max: Int,\n) -&gt; fn(Int) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalidator/int.max\nvalidator/int max","url":"validator/int.html#max"},{"doc":"validator/int","title":"min","content":"pub fn min(\n  error: a,\n  min: Int,\n) -&gt; fn(Int) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalidator/int.min\nvalidator/int min","url":"validator/int.html#min"},{"doc":"validator/int","title":"validator/int","content":"","url":"validator/int.html"},{"doc":"validator/list","title":"every","content":"pub fn every(\n  validator: fn(a) -&gt; Result(b, #(c, List(c))),\n) -&gt; fn(List(a)) -&gt; Result(List(b), #(c, List(c)))\n Validate a list of items.\n\n Run the given validator for each item returning all the errors.\n\n ## Example\n\n\ttype Collection = { Collection(items: List(String) ) }\n\n\tlet list_validator = v_list.every(\n\t\tv_string.min_length(&quot;Must be at least 3&quot;, 3)\n\t)\n\n\tlet validator = fn(collection: Collection) {\n\t\tv.build1(Collection)\n\t\t|&gt; v.validate(collection.items, list_validator)\n\t}\nSynonyms:\nvalidator/list.every\nvalidator/list every","url":"validator/list.html#every"},{"doc":"validator/list","title":"is_not_empty","content":"pub fn is_not_empty(\n  error: a,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate that a list is not empty\nSynonyms:\nvalidator/list.is_not_empty\nvalidator/list is_not_empty","url":"validator/list.html#is_not_empty"},{"doc":"validator/list","title":"max_length","content":"pub fn max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate the max number of items in a list\nSynonyms:\nvalidator/list.max_length\nvalidator/list max_length","url":"validator/list.html#max_length"},{"doc":"validator/list","title":"min_length","content":"pub fn min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate the min number of items in a list\nSynonyms:\nvalidator/list.min_length\nvalidator/list min_length","url":"validator/list.html#min_length"},{"doc":"validator/list","title":"validator/list","content":"","url":"validator/list.html"},{"doc":"validator/option","title":"is_some","content":"pub fn is_some(\n  error: a,\n) -&gt; fn(Option(b)) -&gt; Result(b, #(a, List(a)))\n Validate that a value is not None.\n Returns the value if Some.\n\n ## Example\n\n\ttype PersonInput { PersonInput(name: Option(String)) }\n\n\ttype PersonValid { PersonValid(name: String) }\n\n\tlet validator = fn(person) {\n\t\tv.build1(PersonValid)\n\t\t|&gt; v.validate(person.name, option.is_some(&quot;Name is null&quot;))\n\t}\n\nSynonyms:\nvalidator/option.is_some\nvalidator/option is_some","url":"validator/option.html#is_some"},{"doc":"validator/option","title":"optional","content":"pub fn optional(\n  validator: fn(a) -&gt; Result(a, #(b, List(b))),\n) -&gt; fn(Option(a)) -&gt; Result(Option(a), #(b, List(b)))\n Validate an optional value.\n\n Run the validator only if the value is Some.\n If the value is None then just return None back.\n\n ## Example\n\n\tlet validator = fn(person) {\n\t\tv.build1(PersonValid)\n\t\t|&gt; v.validate(\n\t\t\tperson.name,\n\t\t\toption.optional(string.min_length(&quot;Short&quot;, 3))\n\t\t)\n\t}\n\nSynonyms:\nvalidator/option.optional\nvalidator/option optional","url":"validator/option.html#optional"},{"doc":"validator/option","title":"validator/option","content":"","url":"validator/option.html"},{"doc":"validator/string","title":"is_email","content":"pub fn is_email(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate if a string is an email.\n\n This checks if a string follows a simple pattern `_@_`.\nSynonyms:\nvalidator/string.is_email\nvalidator/string is_email","url":"validator/string.html#is_email"},{"doc":"validator/string","title":"is_float","content":"pub fn is_float(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Float, #(a, List(a)))\n Validate if a string parses to an Float. Returns the Float if so.\nSynonyms:\nvalidator/string.is_float\nvalidator/string is_float","url":"validator/string.html#is_float"},{"doc":"validator/string","title":"is_int","content":"pub fn is_int(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Int, #(a, List(a)))\n Validate if a string parses to an Int. Returns the Int if so.\nSynonyms:\nvalidator/string.is_int\nvalidator/string is_int","url":"validator/string.html#is_int"},{"doc":"validator/string","title":"is_not_empty","content":"pub fn is_not_empty(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate if a string is not empty\nSynonyms:\nvalidator/string.is_not_empty\nvalidator/string is_not_empty","url":"validator/string.html#is_not_empty"},{"doc":"validator/string","title":"max_length","content":"pub fn max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate the max length of a string\nSynonyms:\nvalidator/string.max_length\nvalidator/string max_length","url":"validator/string.html#max_length"},{"doc":"validator/string","title":"min_length","content":"pub fn min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate the min length of a string\nSynonyms:\nvalidator/string.min_length\nvalidator/string min_length","url":"validator/string.html#min_length"},{"doc":"validator/string","title":"validator/string","content":"","url":"validator/string.html"}]);